/*
  Copyright 2008 CNRS
  Author: Florent Lamiraux
*/

#include "kppInterfaceDisplay/kppCommandUpdateGenoMTraj.h"

#include "KineoModel/kppComponentParameter.h"
#include "KineoModel/kppValue.h"
#include "KineoModel/kppComponentClassFilter.h"

#include "posterLib.h"
#include "trajPoster.h"

// Select level of verbosity at configuration.
#if DEBUG==2
#define ODEBUG2(x) std::cout << "CkppCommandUpdateGenoMTraj:" << x << std::endl
#define ODEBUG1(x) std::cerr << "CkppCommandUpdateGenoMTraj:" << x << std::endl
#elif DEBUG==1
#define ODEBUG2(x)
#define ODEBUG1(x) std::cerr << "CkppCommandUpdateGenoMTraj:" << x << std::endl
#else
#define ODEBUG2(x)
#define ODEBUG1(x)
#endif

CkppCommandUpdateGenoMTrajShPtr CkppCommandUpdateGenoMTraj::create(CkppInterfaceDisplay *inKppInterface)
{
  CkppCommandUpdateGenoMTraj* commandPtr = new CkppCommandUpdateGenoMTraj(inKppInterface);
  CkppCommandUpdateGenoMTrajShPtr commandShPtr(commandPtr);

  if (commandPtr->init(commandShPtr) == KD_ERROR) {
    commandShPtr.reset();
  }
  return commandShPtr;
}

CkppCommandUpdateGenoMTrajShPtr CkppCommandUpdateGenoMTraj::createCopy(const CkppCommandUpdateGenoMTrajConstShPtr& inCommand)
{
  CkppCommandUpdateGenoMTraj* commandPtr = new CkppCommandUpdateGenoMTraj(*inCommand);
  CkppCommandUpdateGenoMTrajShPtr commandShPtr(commandPtr);

  if (commandPtr->init(commandShPtr) == KD_ERROR) {
    commandShPtr.reset();
  }
  return commandShPtr;
}

CkppCommandUpdateGenoMTraj::~CkppCommandUpdateGenoMTraj()
{
}

bool CkppCommandUpdateGenoMTraj::isUndoable() const
{
  return true;
}

CkppCommandShPtr CkppCommandUpdateGenoMTraj::clone() const
{
  return CkppCommandUpdateGenoMTraj::createCopy(attWeakPtr.lock());
}
  
unsigned int CkppCommandUpdateGenoMTraj::countParameters() const
{
  return PARAMETER_COUNT;
}
  
CkppParameterConstShPtr CkppCommandUpdateGenoMTraj::parameter(unsigned int inRank) const
{
  CkppParameterShPtr result;
  CkppComponentShPtr nullComponent;

  switch(inRank) {   
  case PATH:
    result = CkppComponentParameter::create("selected path", CkppComponentClassFilter<CkppPathComponent>());
    break;
  default:
    KIT_ASSERT( false );
  }
  return result;
}

CkppCommandUpdateGenoMTraj::CkppCommandUpdateGenoMTraj(CkppInterfaceDisplay *inKppInterface) :
  attKppInterface(inKppInterface)
{
}

CkppCommandUpdateGenoMTraj::CkppCommandUpdateGenoMTraj(const CkppCommandUpdateGenoMTraj& inCommand) :
  CkppCommand(inCommand), attKppInterface(inCommand.attKppInterface)
{
}

ktStatus CkppCommandUpdateGenoMTraj::init(const CkppCommandUpdateGenoMTrajWkPtr& inWeakPtr)
{
  if (CkppCommand::init(inWeakPtr) == KD_ERROR) {
    return KD_ERROR;
  }
  attWeakPtr = inWeakPtr;
  return KD_OK;
}

ktStatus CkppCommandUpdateGenoMTraj::doExecute()
{
  // Get robot
  CkppComponentShPtr component(paramValue(parameter(PATH)).componentValue());
  CkppPathComponentShPtr pathComponent = KIT_DYNAMIC_PTR_CAST(CkppPathComponent, component);

  if (attKppInterface->updatePathFromPoster(pathComponent->kwsPath()) != KD_OK) {
    return KD_ERROR;
  }
  return KD_OK;
}

